import java.io.*;
import java.net.*;
import java.util.*;

class MainClient  {

    BufferedReader br;
    PrintStream ps;
    BufferedReader consoleIn;
    Socket sockComm;
    String id;
    
    public MainClient(String serverIp, int serverPort) throws IOException {
	sockComm = new Socket(serverIp, serverPort);
	ps = new PrintStream(sockComm.getOutputStream());
	br = new BufferedReader(new InputStreamReader(sockComm.getInputStream()));
	// get my id
	ps.println("0");
	id = br.readLine();
    }
 
    public void requestLoop() throws IOException { 
 
	boolean stop = false;
	String reqLine = null;
	BufferedReader consoleIn = null;
	String[] reqParts = null;
 
	consoleIn = new BufferedReader(new InputStreamReader(System.in));
 
	while (!stop) {
	    System.out.print("Client> ");
	    reqLine = consoleIn.readLine();
	    reqParts = reqLine.split(" ");
 
	    if ("storetime".equals(reqParts[0])) {
		// check nb parameters
		if (reqParts.length == 2) {
		    processRequestStoreTime(reqParts[1]);
		}
	    }
	    else if ("average".equals(reqParts[0])) {
		// check nb parameters
		if (reqParts.length == 1) {
		    processRequestAverage();
		}		
	    }
	    else if ("find".equals(reqParts[0])) {
		// check nb parameters
		if (reqParts.length == 2) {
		    processRequestFind(reqParts[1]);
		}		
	    }
	    else if ("minmax".equals(reqParts[0])) {
		// check nb parameters
		if (reqParts.length == 1) {
		    processRequestMinMax();
		}		
	    }
	    else if ("last".equals(reqParts[0])) {
		/* NB: modified version compared to what is asked
		   params contains the list of ids for which the last hour is asked,
		   if no params: ask for all clients.
		*/

		// check nb parameters
		if (reqParts.length == 1) {
		    processRequestLast(null);
		}		
		else if (reqParts.length == 2) {
		    processRequestLast(reqParts[1]);
		}		
	    }	   	    
	    else if ("quit".equals(reqParts[0])) {
		ps.println("quit");
		stop = true;
	    }
	}
    }

    public void processRequestStoreTime(String params) throws IOException {
	String[] lstParams = params.split(",");
	if (lstParams.length != 3) {
	    System.out.println("invalid number of parameters");
	    return;
	}
	String req = "1 "+id+" "+lstParams[0]+" "+lstParams[1]+" "+lstParams[2];
	ps.println(req);
	String ack = br.readLine();
	if ("ERR_ID".equals(ack)) {
	    System.out.println("invalid client id");
	}
	else if ("ERR_PARAM".equals(ack)) {
	    System.out.println("invalid parameter value");
	}
	else {
	    System.out.println("success");
	}
	
    }
    
    public void processRequestAverage() throws IOException {
	String req = "2 "+id;
	ps.println(req);
	String ack = br.readLine();
	if ("ERR_ID".equals(ack)) {
	    System.out.println("invalid client id");
	}
	else {
	    String ans = br.readLine();
	    System.out.println("average = "+ans);
	}
    }
    
    public void processRequestFind(String params) throws IOException {
	String[] lstParams = params.split(",");
	if (lstParams.length != 4) {
	    System.out.println("invalid number of parameters");
	    return;
	}
	String req = "3 "+id+" "+lstParams[0]+" "+lstParams[1]+" "+lstParams[2]+" "+lstParams[3];
	ps.println(req);
	String ack = br.readLine();
	if ("ERR_ID".equals(ack)) {
	    System.out.println("invalid client id");
	}
	else if ("ERR_PARAM".equals(ack)) {
	    System.out.println("invalid parameter value");
	}
	else {
	    String ans = br.readLine();
	    System.out.println(ans);
	}
    }
    
    public void processRequestMinMax() throws IOException {
	String req = "4 "+id;
	ps.println(req);
	String ack = br.readLine();
	if ("ERR_ID".equals(ack)) {
	    System.out.println("invalid client id");
	}
	else {
	    String ans = br.readLine();
	    String[] ansParts = ans.split(" ");
	    System.out.println("min = "+ansParts[0]+", max = "+ansParts[1]);
	}	
    }
    
    public void processRequestLast(String params) throws IOException {

	boolean forAll = true;
	String req = "5 "+id;
	
	if (params == null) {
	    forAll = false;
	    req += " -1";
	}
	else {
	    String[] lstParams = params.split(",");
	    for(int i=0;i<lstParams.length;i++) {
		req+= " "+lstParams[i];
	    }
	}
	ps.println(req);
	String ack = br.readLine();
	if ("ERR_ID".equals(ack)) {
	    System.out.println("invalid client id");
	}
	else if ("ERR_PARAM".equals(ack)) {
	    System.out.println("invalid parameter value");
	}
	else {
	    String ans = br.readLine();
	    System.out.println(ans);
	}	
    }   
}
