import java.io.*;
import java.net.*;
import java.util.*;
 
class MainServer  {
 
    BufferedReader br; 
    PrintStream ps;
    ServerSocket sockConn;
    Socket sockComm;
    int idClient;
    Map<Integer,ArrayList<Time>> map;
    
 
    public MainServer(int port) throws IOException {
	idClient = 0;
	sockConn = new ServerSocket(port);
	map = new HashMap<>();
    }
 
    public void mainLoop() throws IOException {
	while(true) {
	    sockComm = sockConn.accept();
	    try {
		br = new BufferedReader(new InputStreamReader(sockComm.getInputStream()));
		ps = new PrintStream(sockComm.getOutputStream());
		idClient += 1;
		System.out.println("client "+idClient+" connected");
		ArrayList<Time> lst = new ArrayList<>();
		map.put(idClient,lst);
		requestLoop();
		br.close(); // close stream => close connection
		ps.close();
	    }
	    catch(IOException e) {
		System.out.println("client disconnected");
	    }
	}
    }
 
    public void requestLoop() throws IOException {
 
	String req = "";
	boolean stop = false;
	boolean idReqDone = false; // to check that first request is 0
 
	while(!stop) {
	    req = br.readLine();
	    if (req == null) return; // stop loop : suppose that client disconnected
	    if (req.isEmpty()) continue; // empty req => do nothing special
	    String[] reqParts = req.split(" ");
	    
	    // first check if it is req 0
	    if ( "0".equals(reqParts[0])) {       
		ps.println(idClient);
		idReqDone = true;
		continue;
	    }
	    
	    // then, detect if id has been sent with a correct value
	    if (reqParts.length < 2) {
		ps.println("ERR_NB_PARAM");
		continue;
	    }
	    int idSent = -1;
	    try {
		idSent = Integer.parseInt(reqParts[1]);
	    }
	    catch(NumberFormatException e) {}

	    if ((idSent == -1) || (idSent != idClient)) {
		ps.println("ERR_ID");
		continue;
	    }	        
 
	    // storetime request
	    if ("1".equals(reqParts[0])) {
		if (!idReqDone) return; // stop requestLoop so that it stops conn.
		if (reqParts.length != 5) { // check number of params
		    ps.println("ERR_NB_PARAM");
		}
		else {
		    processRequestStoreTime(reqParts[2], reqParts[3], reqParts[4]);
		}
	    }
	    // average request
	    else if ("2".equals(reqParts[0])) {
		if (!idReqDone) return; // stop requestLoop so that it stops conn.
		if (reqParts.length != 2) { // check number of params
		    ps.println("ERR_NB_PARAM");
		}
		else {
		    processRequestAverage();
		}
	    }
	    // find request
	    else if ("3".equals(reqParts[0])) {
		if (!idReqDone) return; // stop requestLoop so that it stops conn.
		if (reqParts.length != 6) { // check number of params
		    ps.println("ERR_NB_PARAM");
		}
		else {
		    processRequestFind(reqParts[2], reqParts[3], reqParts[4], reqParts[5]);
		}
	    }
	    // minmax request
	    else if ("4".equals(reqParts[0])) {
		if (!idReqDone) return; // stop requestLoop so that it stops conn.
		if (reqParts.length != 2) { // check number of params
		    ps.println("ERR_NB_PARAM");
		}
		else {
		    processRequestMinMax();
		}
	    }
	    // last request
	    else if ("5".equals(reqParts[0])) {
		if (!idReqDone) return; // stop requestLoop so that it stops conn.
		if (reqParts.length < 3) { // check number of params
		    ps.println("ERR_NB_PARAM");
		}
		else {
		    processRequestLast(reqParts); // more practical to give the whole array
		}
	    }	    
	    /* etc. with other requests */
	    else if ("quit".equals(reqParts[0])) {
		stop = true;
	    }
	}
    }  
 
    public void processRequestStoreTime(String hS, String mS, String sS) throws IOException {
	int h,m,s;
	// add here other params to translate into numbers
	try {
	    h = Integer.parseInt(hS);
	    m = Integer.parseInt(mS);
	    s = Integer.parseInt(sS);	    
	}
	catch(NumberFormatException e) { 
	    ps.println("ERR_PARAM");
	    return; 
	}
	// add here other tests on other parameters
	if ((h<0)||(h>24)||(m<0)||(m>59)||(s<0)||(s>59)) {
	    ps.println("ERR_PARAM");
	    return; 	    
	} 
	// if all ok, 
	ps.println("OK");
	// process the request and send the result to the client
	ArrayList<Time> lst = map.get(idClient);
	lst.add(new Time(h,m,s));
    }

    public void processRequestAverage() throws IOException {
	// if all ok, 
	ps.println("OK");
	
	ArrayList<Time> lst = map.get(idClient);
	double moy = 0.0;
	for(Time t : lst) {
	    moy += t.toSeconds();
	}
	moy /= lst.size();
	
	int[] tab = Time.fromSeconds((int)moy);
	ps.println(tab[0]+":"+tab[1]+":"+tab[2]);
    } 

    public void processRequestFind(String hS, String mS, String sS, String gapS) throws IOException {
	int h,m,s,gap;
	// add here other params to translate into numbers
	try {
	    h = Integer.parseInt(hS);
	    m = Integer.parseInt(mS);
	    s = Integer.parseInt(sS);
	    gap = Integer.parseInt(gapS);
	}
	catch(NumberFormatException e) { 
	    ps.println("ERR_PARAM");
	    return; 
	}
	// add here other tests on other parameters
	if ((h<0)||(h>24)||(m<0)||(m>59)||(s<0)||(s>59)) {
	    ps.println("ERR_PARAM");
	    return; 	    
	} 
	// if all ok, 
	ps.println("OK");
	Time tt = new Time(h,m,s);
	// process the request and send the result to the client
	ArrayList<Time> lst = map.get(idClient);
	for(Time t : lst) {
	    if (t.closeTo(tt,gap)) {
		ps.println("true");
		return;
	    }
	}
	ps.println("false");
    }

    public void processRequestMinMax() throws IOException {
	// if all ok, 
	ps.println("OK");

	int hMin=23*3600+59*60+59;
	int hMax=0;
	
	ArrayList<Time> lst = map.get(idClient);
	for(Time t : lst) {
	    int s = t.toSeconds();
	    if (s < hMin) hMin = s;
	    if (s > hMax) hMax = s;
	}
	int[] tabMin = Time.fromSeconds(hMin);
	int[] tabMax = Time.fromSeconds(hMax);
	ps.println(tabMin[0]+":"+tabMin[1]+":"+tabMin[2]+" "+tabMax[0]+":"+tabMax[1]+":"+tabMax[2]);
    } 

    public void processRequestLast(String[] params) throws IOException {
	int firstId;
	int id;
	ArrayList<Integer> lst = new ArrayList<>();
	try {
	    firstId = Integer.parseInt(params[2]);
	}
	catch(NumberFormatException e) { 
	    ps.println("ERR_PARAM");
	    return; 
	}
	if (firstId == -1) {
	    // take all client except idClient
	    Set<Integer> set = map.keySet();
	    for(Integer i : set) {
		if (i != idClient) {
		    lst.add(i);
		}
	    }
	}
	else {
	    lst.add(firstId);
	    try {
		for(int i=3;i<params.length;i++) {
		    id = Integer.parseInt(params[i]);
		    lst.add(id);
		}
	    }
	    catch(NumberFormatException e) { 
		ps.println("ERR_PARAM");
		return; 
	    }
	}
	
	// if all ok, 
	ps.println("OK");

	String ans="";
	for(Integer i : lst) {
	    ArrayList<Time> tl = map.get(i);
	    if (tl != null) {
		ans += "client "+i+" - ";	   
		Time tt = tl.get(tl.size()-1);
		ans += tt.getH()+":"+tt.getM()+":"+tt.getS()+" | ";
	    }
	}
	ps.println(ans);
    }
    
}
